import { IPlatformComponent, OFFICIAL_LIB_KEY } from '@cloudbase/cals';
import { app as _app, IAppAPI } from '../app/global-api';
import { auth } from '@cloudbase/weda-client';
import { inWxWork } from './sso/helper';
import { tryWecomLogin } from './sso/init';
import { isEqual as lodashIsEqual } from 'lodash-es';

const { loginScope, getAccessToken } = auth;

/**
 * All data bindings are generated as functions: (forItems) => any
 * @param {*} dataBinds
 * @param {*} forItems
 */
export function resolveDataBinds(dataBinds, forItems, codeContext, scopeContext, throwError, dataContext, $w) {
  const resolvedProps = {};
  for (const prop in dataBinds) {
    let fn = dataBinds[prop];
    try {
      if (codeContext?.$WEAPPS_COMP) {
        fn = fn.bind(codeContext.$WEAPPS_COMP);
      }
      resolvedProps[prop] = fn(forItems, codeContext?.event, dataContext, $w, scopeContext);
    } catch (e) {
      console.error('Error resolving data binding', prop, dataBinds[prop], e);
      if (throwError) {
        throw e;
      }
    }
  }
  return resolvedProps;
}

/**
 * 对函数进行批量绑定
 * @param {*} funcObj
 * @param {*} context
 */
export function bindFuncObj(funcObj = {}, context) {
  return Object.keys(funcObj).reduce((obj, fnName) => {
    obj[fnName] = funcObj[fnName]?.bind?.(context);
    return obj;
  }, {});
}

export function kebabCase(str) {
  return str.replace(/[A-Z]/g, (match) => {
    return `-${match.toLowerCase()}`;
  });
}

export function camelcase(str, firstUpperCase = false) {
  str = str.replace(/[_-]([a-z])/g, (l) => {
    return l[1].toUpperCase();
  });

  if (firstUpperCase) str = str.charAt(0).toUpperCase() + str.slice(1);

  return str;
}

export const isEmptyObj = (obj) => {
  if (!isPlainObject(obj)) {
    return false;
  }
  for (const i in obj) {
    if (Object.prototype.hasOwnProperty.call(obj, i)) {
      return false;
    }
  }
  return true;
};

export function isPlainObject(src) {
  return Object.prototype.toString.call(src) === '[object Object]';
}

/**
 * @deprecated
 * 历史静态配置，不再新增
 */
const SCOPE_SLOT_COMPONENT_LIB = ['tea_basis', 'tea_shop', 'crm_basis'];

/**
 * @deprecated
 * 历史静态配置，不再新增
 */
const SCOPE_SLOT_MAP = SCOPE_SLOT_COMPONENT_LIB.reduce((map, lib) => {
  map[`${lib}:TableMatching`] = {
    headerSlot: true,
    recordSlot: true,
  };

  map[`${lib}:TableExpanded`] = {
    headerSlot: true,
    recordSlot: true,
  };

  map[`${lib}:TabsTable`] = {
    headerSlot: true,
    recordSlot: true,
  };

  map[`${lib}:Table`] = {
    headerSlot: true,
    recordSlot: true,
  };

  return map;
}, {});

export function isSlot(comp: IPlatformComponent) {
  return !comp.module && !comp.component;
}

/**
 * 判断是否为作用域插槽
 * 1、根据组件dataForm是否有属性包含 x-slot-scope 属性
 * 2、保持历史穷举法不变
 */
export function getScopeSlotMeta(
  comp: {
    module: string;
    component: string;
  },
  slot: string,
  context?: { componentsInfoMap: any } | any,
): { isScopeSlot: boolean; config?: any; useScopeWidgetData?: boolean } {
  const sourceKey = comp.module && comp.component && `${comp.module}:${comp.component}`;
  const map = SCOPE_SLOT_MAP[sourceKey];
  const res = {
    isScopeSlot: map?.[slot] || false,
    useScopeWidgetData: false,
    config: undefined,
  };

  try {
    const info = context?.componentsInfoMap?.[sourceKey];
    const { dataForm } = info || {};
    if (typeof dataForm === 'object') {
      // eslint-disable-next-line no-restricted-syntax
      for (const prop in dataForm) {
        if (slot.startsWith(`${prop}_`) && dataForm[prop]?.['x-slot-scope']) {
          res.isScopeSlot = true;
          res.useScopeWidgetData = true;
          res.config = { ...dataForm[prop] };

          if (sourceKey === 'CLOUDBASE_BUSSINESS:ModelTable') {
            res.useScopeWidgetData = false;
          } else if (comp.module === OFFICIAL_LIB_KEY || comp.module === 'weda') {
            let gt = false;
            const properties = info.properties?.properties || info.properties;
            if (comp.component === 'WdTable' && properties.records) {
              gt = true;
            }

            if (!gt) {
              res.useScopeWidgetData = false;
            }
          }

          /**
           * 首先 版本号确定使用新版
           * 其次 若无 for 配置 默认补足配置
           */
          if (res.useScopeWidgetData) {
            if (
              (sourceKey === `${OFFICIAL_LIB_KEY}:WdTable` || sourceKey === `weda:WdTable`) &&
              prop === 'cell' &&
              !res.config['x-slot-directives']
            ) {
              res.config['x-slot-directives'] = {
                for: 'records',
                /**
                 * 没想好是记录 key 标识，用 find 去找
                 * 还是直接用 index 来
                 */
                _index: 'recordIndex',
              };
            }
          }

          break;
        }
      }
    }
  } catch (e) {}

  return res;
}

export function findLoginPage(app = _app) {
  const { pages = [] } = app.__internal__.getConfig();
  return pages.find((item) => item.type === 'login');
}

let _AUTH_CONFIG_CACHE = null;
export async function getAuthConfig(app = _app) {
  if (_AUTH_CONFIG_CACHE) {
    return _AUTH_CONFIG_CACHE;
  }
  try {
    const res = await app.cloud.callWedaApi({
      action: 'DescribeRuntimeResourceStrategy',
      data: {
        ResourceType: 'app',
        ResourceId: app.id,
      },
    });
    const settingData = {};
    // 云api不支持map只能传字符串，需要转换
    res.forEach((item) => {
      settingData[item.Key] = ['AllowRegister', 'NeedLogin'].includes(item.Key) ? item.Value === '1' : item.Value;
    });
    _AUTH_CONFIG_CACHE = settingData;
    return _AUTH_CONFIG_CACHE;
  } catch (e) {
    return {
      NeedLogin: false,
      RejectStrategy: 'show_warning',
    };
  }
}

let _AUTH_CACHE_MAP = {};
async function getAccessPermission(app, appId, pageId) {
  const cacheKey = `${appId}-${pageId}`;
  if (_AUTH_CACHE_MAP[cacheKey] !== undefined) {
    return _AUTH_CACHE_MAP[cacheKey];
  }

  const res = await app.cloud.callWedaApi({
    action: 'DescribeResourcesPermission',
    data: {
      ResourceType: `app`,
      ResourceIdList: [cacheKey],
      AppResourceId: appId,
    },
  });
  if (Array.isArray(res)) {
    const resourceInfo = res.find((i) => i.ResourceId === cacheKey);
    const isAccess = !!resourceInfo?.IsAccess;
    _AUTH_CACHE_MAP[cacheKey] = isAccess;
    return isAccess;
  } else {
    throw new Error(`鉴权接口返回参数不正确 - ${JSON.stringify(res)}`);
  }
}

export async function checkAnonymous() {
  // 用户是否非匿名登录前端判断条件：有accessToken且scope不为anonymous
  // 此分支逻辑本不应该前端判断是否登录，历史原因后端短期内搞不定，后续后端优化后删除
  let isAnonymous = true;
  try {
    const [scope, { accessToken }] = await Promise.all([loginScope(), getAccessToken()]);
    if (accessToken && scope !== 'anonymous') {
      isAnonymous = false;
    }
  } catch (e) {}
  return isAnonymous;
}

export function formatErrorMsg(e) {
  let msg = e?.error_description || e?.message;
  const uri = e?.error_uri;
  msg = `${msg}${uri ? `-${uri}` : ''}`;
  if (msg.startsWith('Failed to fetch')) {
    msg = `网络故障, 请检查本地网络连接是否正常(${msg})`;
  }
  return msg;
}

/**
 * 检查页面权限
 **/
export async function checkAuth(app, appId, $page, generateLoginUrl?: GenerateLoginUrlType) {
  app.showNavigationBarLoading();
  const { loginConfigVersion } = app.__internal__.getConfig();
  const requestList = [getAccessPermission(app, appId, $page.id)];
  // 有登录配置版本号时，才拉取自定义登录配置进行校验
  if (loginConfigVersion) {
    requestList.push(getAuthConfig(app));
    requestList.push(checkAnonymous());
  }
  try {
    const [isAccess, authConfig, isAnonymous] = await Promise.all(requestList);
    app.hideNavigationBarLoading();

    if (loginConfigVersion && isAnonymous) {
      if (authConfig.NeedLogin || (!isAccess && authConfig.RejectStrategy === 'to_login')) {
        redirectToLogin(generateLoginUrl);
        return false;
      }
    }

    if (!isAccess) {
      app.showToast({
        title: '页面无访问权限',
        icon: 'error',
      });
    }
    return isAccess;
  } catch (e) {
    app.showModal({
      title: '页面鉴权失败',
      content: formatErrorMsg(e),
      showCancel: false,
    });
    // @ts-ignore
    window._aegis?.error?.('CUSTOM_ERROR:', e);
    return false;
  }
}

export type GenerateLoginUrlType = (ctx: { app: IAppAPI }, originFn: () => string) => string;

export const defaultGenerateLoginUrl: GenerateLoginUrlType = () => {
  return getLoginUrl();
};

function getLoginUrl() {
  const app = _app;
  const { envId, id, clientId, loginConfigVersion } = app.__internal__.getConfig();
  return `${location.origin}/__auth/?redirect_uri=${encodeURIComponent(
    location.href,
  )}&app_id=${id}&env_id=${envId}&client_id=${clientId}&config_version=${loginConfigVersion}`;
}

export async function redirectToLogin(generateLoginUrl: GenerateLoginUrlType) {
  // 去登录则清空权限缓存。
  _AUTH_CACHE_MAP = {};
  const app = _app;

  const { loginConfigVersion } = app.__internal__.getConfig();
  // 有登录配置版本号时，自定义登录开启，则跳转到自定义登录页面
  if (loginConfigVersion) {
    const url = generateLoginUrl({ app }, getLoginUrl);
    const alternative = function alternative() {
      app.redirectTo({
        mode: 'web',
        params: {},
        url,
      });
    };
    if (ENABLE_WECOM) {
      tryWecomLogin(new URL(url), alternative);
    } else {
      alternative();
    }
  } else {
    app.showToast({
      title: '用户未登录',
      icon: 'error',
    });
  }
}

export function generateDataContext(widget) {
  const dataContext = {};
  while (widget?._scope) {
    const current = widget;
    // 此处采用代理的方式，是为了可以获取到最新的 _scope.dataContext 防止 dataContext 引用被重新赋值
    Object.defineProperty(dataContext, current._scope.id, {
      get() {
        return current._scope.dataContext;
      },
    });
    widget = widget.parent;
  }
  return dataContext;
}

export let ENABLE_OBSERVER_REF = false;
export function setEnableObserverRef(value) {
  ENABLE_OBSERVER_REF = value;
}

export let ENABLE_PAGE_ROOT_ID = false;
export function setEnablePageRoot(value) {
  ENABLE_PAGE_ROOT_ID = value;
}
export let ENABLE_WECOM = false;
export function setEnableWecom(value) {
  ENABLE_WECOM = value;
}

/**
 * 是否开启原地企微登录
 * 1. 针对单环境白名单
 * 2. 在企微环境打开
 * 3. 自定义应用（即：非企业工作台)
 */
const WECOM_ENVID_LIST = [
  'lowcode-4gzdsu5k66946492',
  'lowcode-2gay8jgh25c7b1cf',
  'lowcode-4g9b78bx94404d85',
  'lowcode-8gmx7bat3135e2de',
];
export function isEnableWecom(envId) {
  const adminPortal = simpleCheckAdminPortal();
  if (inWxWork() && !adminPortal && WECOM_ENVID_LIST.includes(envId)) {
    return true;
  }
}

export function serde(a) {
  return JSON.parse(JSON.stringify(a));
}

const isObj = (a) => typeof a === 'object' && a !== null;

export function isEqual(a, b, shouldSerde = false) {
  if (shouldSerde && isObj(a) && isObj(b)) {
    // for Proxy created by mbox
    return lodashIsEqual(serde(a), serde(b));
  }
  return lodashIsEqual(a, b);
}

const _REPORTED = {};
export function reportTime(tag, time, only = false) {
  const excludes = ['app-nvzcvt10', 'app-msa2ihs9'];
  if (!window._aegis || !tag || simpleCheckAdminPortal() || !window._aegis_inited || excludes.includes(_app.id)) {
    return;
  }
  if (window._WedaHostConfig && !window._WedaHostConfig['_REPORTED']) {
    window._WedaHostConfig['_REPORTED'] = _REPORTED;
  }

  const CACHE_MAP = window._WedaHostConfig?.['_REPORTED'] || _REPORTED;

  if (only && CACHE_MAP[tag]) {
    return;
  }
  CACHE_MAP[tag] = true;

  const { envVersion } = _app.__internal__?.getConfig?.() || {};

  try {
    let t = time === undefined ? performance?.now?.() || Date.now() - window._aegis_inited : time;
    window._aegis.reportTime({
      name: tag,
      duration: t,
      ext2: envVersion,
    });
    if (t > 5000 && performance?.getEntries) {
      window._aegis.report({
        msg: performance.getEntries(),
        ext2: envVersion,
        trace: 'trace',
      });
    }
  } catch (e) {
    console.error(e);
  }
}

export function reportEvent(tag) {
  const { envVersion } = _app.__internal__?.getConfig?.() || {};
  try {
    window._aegis.reportEvent({
      name: tag,
      ext2: envVersion,
    });
  } catch (e) {
    console.error(e);
  }
}

export function simpleCheckAdminPortal() {
  return window._WedaHostConfig && window.__POWERED_BY_QIANKUN__;
}

export function getNetworkType(window) {
  const ua = window?.navigator?.userAgent || '';
  let networkTypeStr = ua.match(/NetType\/\w+/)
    ? ua
        .match(/NetType\/\w+/)?.[0]
        ?.toLowerCase()
        ?.replace('nettype/', '')
    : window?.navigator?.connection?.effectiveType?.toLowerCase() || '';
  const types = ['2g', '3g', '4g', '5g', 'wifi'];
  for (const type of types) {
    if (networkTypeStr.includes(type)) return type;
  }
  return networkTypeStr ? 'other' : '';
}
