import { resolveDataBinds, reportEvent } from '../common';
import { set as lodashSet } from 'lodash-es';
import { app } from '../../app/global-api';
import { runtime } from '../adapter';
import { untracked } from 'mobx';
import { generateWidgetAPIContext } from '../widgets';

const DEFAULT_MAX_TIMEOUT = 10 * 1000;

class Event {
  type: string;
  detail: any;
  /**
   * 类别应当为 typeof UserWidget
   * 当前类别为 typeof $page.widgets.xxxx
   * 添加上报确定用量
   */
  currentTarget: any /* typeof UserWidget */;
  target?: any /* typeof UserWidget */;
  /**
   * 内部实现
   * 外部不应该进行访问
   */
  _isCapturePhase: boolean;
  origin: Event;
  constructor({
    type = '',
    detail = undefined,
    currentTarget = undefined,
    target = undefined,
    _isCapturePhase = false,
    origin,
  }: {
    type: string;
    currentTarget: any;
    target?: any;
    detail?: any;
    _isCapturePhase?: boolean;
    origin?: Event;
  }) {
    this.type = type;
    this.detail = detail;
    this.currentTarget = currentTarget;
    this.target = target;
    this._isCapturePhase = _isCapturePhase;
    this.origin = origin;
    return new Proxy(this, {
      get(target, prop) {
        switch (prop) {
          case 'name': {
            console.warn('[deprecated] event.name 将在未来版本放弃支持，请使用 event.type 替代');
            return target.type;
          }
          case 'origin':
          case 'target':
          case 'currentTarget': {
            return target[prop]
              ? new Proxy(target[prop], {
                  get(target, p) {
                    reportEvent(`event.${prop}`);
                    return target[p];
                  },
                })
              : target[prop];
            break;
          }
        }

        return target[prop];
      },
    });
  }
}

function emitEvent(trigger, listeners = [], args, scopeContext = {}, dataContext, $w, pageId = '') {
  const targetListeners = listeners.filter((l) => l.trigger === trigger);
  const currentEvent = args?.event;

  async function processListener(listener) {
    // 当前非捕获Event，再判断冒泡行为
    if (!currentEvent?._isCapturePhase && listener.noPropagation) {
      args?.originalEvent?.stopPropagation && args?.originalEvent?.stopPropagation();
    }
    // 判断捕获的执行，只有执行的捕获与配置的捕获一致时。才会执行。
    // 为逻辑分支 额外新增处理

    const nextEvent = new Event({
      ...currentEvent,
      type: '',
      detail: '',
      origin: currentEvent,
    });

    if ((listener?.isCapturePhase || false) === (currentEvent?._isCapturePhase || false)) {
      if (pageId && $w?.page?.id && pageId !== $w?.page?.id) {
        console.error(`Action error: [${listener.trigger}] 页面生命周期结束，链式调用中断`);
        return;
      }
      if (listener?.sourceKey === 'platform:utils.If') {
        try {
          const res = await invokeListener(listener, args, scopeContext, dataContext, $w);
          nextEvent.type = `${listener.key}.${res ? 'success' : 'fail'}`;
        } catch (e) {
          nextEvent.type = `${listener.key}.fail`;
        }
        nextEvent.detail = currentEvent.detail;
      } else {
        try {
          const res = await invokeListener(listener, args, scopeContext, dataContext, $w);
          nextEvent.type = `${listener.key}.success`;
          nextEvent.detail = res;
        } catch (e) {
          if (e.message) {
            e.message = `${currentEvent?.currentTarget?.id || ''}[${listener.trigger}] ${e.message}`;
          }

          nextEvent.type = `${listener.key}.fail`;
          nextEvent.detail = e;

          // 之前 invoke 内部catch 了错误，不会抛错
          // throw e
          console.error(`Action error: [${listener.trigger}]`, e);
        }
      }
    }

    emitEvent(
      nextEvent.type,
      listeners,
      {
        ...args,
        event: nextEvent,
      },
      scopeContext,
      dataContext,
      $w,
      pageId,
    );
  }

  if (!targetListeners.length && typeof trigger === 'string' && /\.fail$/.test(trigger)) {
    app.showToast({
      title: `事件响应失败：${currentEvent?.detail}`,
      icon: 'error',
    });
  } else {
    targetListeners.forEach((listener) => {
      processListener(listener);
    });
  }
}

export function generateEmit(
  $w,
  currentWidget = undefined, // page 节点暂未生成
  injectContext = undefined,
  extraListeners = [],
  extraForContext = {},
) {
  return (trigger, listeners = [], eventData, forItems, originalEvent, scopeContext, dataContext, pageId = '') => {
    listeners = listeners.concat(extraListeners);

    if (injectContext?.emit) {
      return injectContext?.emit(trigger, listeners, eventData, forItems, originalEvent, scopeContext, dataContext);
    } else {
      const event = new Event({
        detail: eventData,
        type: trigger,
        target: currentWidget,
        currentTarget: currentWidget,
      });
      const forContext = {
        ...forItems,
        ...extraForContext,
      };
      emitEvent(
        event.type,
        listeners,
        {
          event,
          forItems: forContext,
        },
        scopeContext,
        dataContext,
        untracked(() => {
          return generateWidgetAPIContext($w, currentWidget, forContext, scopeContext);
        }),
        pageId,
      );
    }
  };
}

function invokeListener(
  listener,
  args: {
    forItems: Record<string, any>;
    event: Event;
  },
  scopeContext,
  dataContext,
  $w,
) {
  const { key, instanceFunction, data = {}, dataBinds = {} } = listener;
  if (!instanceFunction) {
    try {
      // @ts-ignore
      window._aegis?.error?.('SYSTEM_ERROR: listener not found', listener);
      console.error('$page current hander', runtime?.['$page']?.handler);
    } catch (e) {}
    throw new Error(`${key} handler 未找到'`);
  }
  // ToDo resolve databinds
  const action = instanceFunction;
  const resolvedData = {
    ...data,
  };
  const resolvedDataBinds = resolveDataBinds(
    dataBinds,
    args.forItems,
    { event: args.event },
    scopeContext,
    true,
    dataContext,
    $w,
  );

  // eslint-disable-next-line no-restricted-syntax
  for (const key in resolvedDataBinds) {
    if (resolvedDataBinds[key] && resolvedDataBinds[key].__type === 'scopedValue') {
      try {
        lodashSet(resolvedData, key, resolvedDataBinds[key].getValue(scopeContext));
      } catch (e) {
        lodashSet(resolvedData, key, '');
      }
    } else {
      lodashSet(resolvedData, key, resolvedDataBinds[key]);
    }
  }
  const params = {
    data: resolvedData,
    ...args,
  };

  return action(params);
}
