import { Router } from './router';

function prefixMsg(msg: string, prefix: string = ' - ') {
  return `${msg ? prefix + msg : ''}`;
}

export function errorPage(router: Router) {
  return (e: HTMLFormElement, err: any) => {
    let title;
    let desc;
    switch (err.error as string) {
      case 'unreachable':
        title = '网络错误';
        desc = `请检查您的网络连接，稍后重试${prefixMsg(e.error_description)}`;
        break;
      case 'permission_denied':
        title = '权限不足';
        desc = `错误详情 - ${err.error_description}`;
        break;
      case '-':
        desc = err.error_description;
        break;
      case 'unauthorized_organization':
        // @todo @nanxi企微场景
        desc = '该应用可能未在您的企业安装，请重新安装，然后5分钟后重试';
        break;
      case '-':
        desc = err.error_description;
        break;
      case 'success':
        e.querySelector('.icon')?.classList.add('hidden');
        title = '操作成功';
        desc = err.error_description;
        break;
      default:
        desc = `错误详情 - ${err.error || err.message}${prefixMsg(err.error_description)}`;
        break;
    }
    if (title) {
      (<HTMLDivElement>e.querySelector('.title')).innerText = title;
    }
    if (desc) {
      (<HTMLDivElement>e.querySelector('.desc')).innerText = desc;
    }
    // 异常上报
    if (router.getUrlParams('redirect_uri')) {
      const redirectBtn = <HTMLDivElement>e.querySelector('.redirect-btn');
      redirectBtn.style.display = 'block';
      redirectBtn.onclick = (e) => {
        e.preventDefault();
        router.redirectURIorBackTo();
      };
    }
    router.loaded();
  };
}

// 登录错误处理
export const parseLoginError = (e: any) => {
  switch (e.error as string) {
    case 'not_found':
      return '用户不存在';
    case 'password_not_set':
      return '当前用户未设置密码，请使用验证码登录或第三方登录方式';
    case 'invalid_password':
      if (e.details && e.details.length > 0) {
        return `密码验证失败, 您还有 ${e.details[0].remaining} 次重试机会`;
      }
      return '密码不正确';
    case 'user_pending':
      return '该用户未激活';
    case 'user_blocked':
      return '该用户被停用';
    case 'invalid_status':
      if (e.details && e.details.length > 0) {
        const retryInSeconds = e.details[0].retry_in;
        const retryInTime = retryInSeconds < 3600 ? `${retryInSeconds / 60}分钟` : `${retryInSeconds / 3600}小时`;
        return `您已经超过了密码最大重试次数，请于 ${retryInTime} 后重试`;
      }
      return '您已经超过了密码最大重试次数，请稍后重试';
    default:
      return '您输入的帐号或密码不正确，请重新输入';
  }
};

// 注册错误处理
export const parseSignUpError = (e: any) => {
  switch (e.error as string) {
    case 'failed_precondition':
      return '您输入的手机号或邮箱已被注册，请使用其他号码';
    default:
      return parseVerifyError(e);
  }
};

// 验证码验证token
export const parseVerifyError = (e: any) => {
  switch (e.error as string) {
    case 'invalid_argument':
      return '您输入的验证码不正确或已过期';
    case 'aborted':
      return '您尝试的次数过多，稍后重试';
    default:
      return parseCommonError(e);
  }
};

// 三方登录表单错误
export const parseProviderSignError = (e: any) => {
  switch (e.error as string) {
    case 'permission_denied':
      return '您当前的会话已过期，请返回重试';
    case 'invalid_argument':
      return '您输入的验证码不正确或已过期';
    case 'aborted':
      return '您尝试的次数过多，稍后重试';
    default:
      return parseCommonError(e);
  }
};

// 重置密码错误
export const parseResetPasswordError = (e: any) => {
  switch (e.error as string) {
    case 'invalid_argument':
      return '您输入的密码不符合要求, 请重新输入';
    default:
      return parseCommonError(e);
  }
};

// 验证码验证token
export const parseProviderTokenError = (e: any) => {
  switch (e.error as string) {
    case 'failed_precondition':
      return `从第三方获取用户信息失败${prefixMsg(e.error_description)}`;
    case 'not_found':
      return `没有找到匹配的用户\n请联系管理员确认您的配置`;
    default:
      return parseCommonError(e);
  }
};

export function parseValidateError(e: any, verificationTarget: string) {
  switch (e.error as string) {
    case 'failed_precondition':
      switch (verificationTarget) {
        case 'USER':
          return '你输入的手机号或邮箱已被注册，请使用其他号码';
        case 'NOT_USER':
          return '该用户已被注册，请使用其他号码';
        default:
          return '短信功能未开启，请联系管理员开启';
      }
    case 'payment_required':
      return '短信包剩余条数不足，请联系管理员充值';
    case 'aborted':
      return '你尝试的次数过多，稍后重试';
    case 'unavailable':
      return `发件服务不可用，请稍后重试${prefixMsg(e.error_description)}`;
    default:
      return parseCommonError(e);
  }
}

export const parseCommonError = (e: any) => {
  let msg: string;
  switch (e.error as string) {
    case 'unreachable':
      msg = `网络错误，请检查您的网络连接，稍后重试${prefixMsg(e.error_description)}`;
      break;
    case 'resource_exhausted':
      return '您尝试过于频繁，请稍后重试';
    default:
      msg = `服务异常, 请稍后重试${prefixMsg(e.error || e.message)}${prefixMsg(e.error_description)}`;
      break;
  }
  return msg;
};
