import React, { useMemo, useState, useEffect } from 'react';
import { Tabs, TabPanel, ConfigProvider, message, Button, Modal, Card } from 'tea-component';
import { useSetState } from 'ahooks';
import { DataItemView } from './View';
import { BaseTable } from '../Table';
import { PageContent, PageNullDb, PageCountDown } from '../PageContent';
import { Title } from '../Space';
import { isProcessHoldOrLocked } from '../Table/FieldRender';
import { SelectableBlock } from '../SelectableBlock';
import { useAuthFields } from '../../hooks';
import { getRecordId, navigateToModelPage, getDataSourceByName, DataSource, isInIde, } from '../../utils';
import { alertErrorMessage } from '../../utils/feedback';
import { DETAIL_BUTTONS, TABLE_BUTTONS } from '../../constants';
import { EnumProvider } from '../../context';
import './index.less';
// Detail和父子表格两个子组件状态提升
export const DetailLoadingContext = React.createContext(null);
/**
 * 详情组件
 */
export const ModelDetail = (props) => {
    const { bindMetadata = {}, // data属性
    buttons = DETAIL_BUTTONS, // 选区按钮集合
    _selectableBlockEvents: selectableBlockEvents, // 选区的事件集合
    id: compId, // 组件的id
    showRelatedTable = true, // 是否展示关联列表
    recordId, // 通过属性传递的数据ID
    ...rest } = props;
    const { datasource = {}, enabledFieldNames } = bindMetadata;
    const { id } = getRecordId(recordId);
    const dbName = datasource === null || datasource === void 0 ? void 0 : datasource.name;
    const datasourceExp = useMemo(() => new DataSource(dbName), [dbName]);
    const [recordData, setRecordData] = useState({
        _id: undefined,
    });
    const [detailLoading, setDetailLoading] = useState(true);
    const [{ showFields, dbTitle, tabs, tabChild }, setState] = useSetState({
        datasourceProfile: {},
        dbTitle: '',
        showFields: [],
        tabs: [],
        tabChild: [],
    });
    useEffect(() => {
        // 子相关列表
        async function getTabChild(tabsList) {
            const tabChildPromise = tabsList.map(async (tab) => {
                const childDbName = tab.id;
                const childDatasource = await getDataSourceByName(childDbName, true);
                const enabledFieldNames = Object.keys((childDatasource === null || childDatasource === void 0 ? void 0 : childDatasource.schema.properties) || {});
                const childDataSourceRelatedField = getChildDataSourceRelatedField(childDatasource, dbName);
                const detailId = id;
                return (React.createElement(TabPanel, { id: tab.id },
                    React.createElement(TabChildRender, { dbName: childDbName, enabledFieldNames: enabledFieldNames, key: childDbName, childDataSourceRelatedField: childDataSourceRelatedField, detailId: detailId, compId: compId, buttons: TABLE_BUTTONS, selectableBlockEvents: selectableBlockEvents })));
            });
            const result = await Promise.all(tabChildPromise);
            return result;
        }
        async function initState() {
            var _a;
            const { showFields } = await useAuthFields(dbName, enabledFieldNames, true);
            const datasourceProfile = await getDataSourceByName(dbName, true);
            // 存在子数据源，添加相关列表 tab
            let tabsList = [];
            let tabChildValue = [];
            if ((_a = datasourceProfile === null || datasourceProfile === void 0 ? void 0 : datasourceProfile.childDataSourceNames) === null || _a === void 0 ? void 0 : _a.length) {
                const dbNames = Array.from(new Set((datasourceProfile === null || datasourceProfile === void 0 ? void 0 : datasourceProfile.childDataSourceNames) || []));
                const tabsListPromise = dbNames.map(async (d) => {
                    const ds = await getDataSourceByName(d, true);
                    return ds ? { id: d, label: (ds === null || ds === void 0 ? void 0 : ds.title) || d } : null;
                });
                tabsList = await Promise.all(tabsListPromise);
                tabsList = tabsList.filter((item) => !!item);
                // 存在被关联的子数据源生成 TabChild
                if (tabsList.length > 0) {
                    tabChildValue = await getTabChild(tabsList);
                }
            }
            setState({
                showFields,
                dbTitle: (datasourceProfile === null || datasourceProfile === void 0 ? void 0 : datasourceProfile.title) || dbName,
                tabs: tabsList,
                tabChild: tabChildValue,
            });
        }
        initState();
    }, [dbName]);
    // 1、编辑器内未绑定数据源时返回
    if (!dbName) {
        return React.createElement(PageNullDb, { ...rest });
    }
    // 2、未找到url上的数据源id时返回
    if (!id && !isInIde()) {
        alertErrorMessage({ message: '数据记录 ID 不存在，加载数据失败' });
        return React.createElement(PageCountDown, { dbName: dbName, description: "\u8BB0\u5F55 ID \u4E0D\u5B58\u5728", ...rest });
    }
    const actionMap = {
        edit: () => {
            if (isInIde()) {
                return alertErrorMessage({ message: '请开启“实时真实预览”后，再进行编辑操作' });
            }
            navigateToModelPage('update', { dbName, id });
        },
        del: () => {
            const modal = Modal.show({
                className: 'modal-table-delete',
                caption: '操作确认',
                onClose: () => modal.destroy(),
                children: (React.createElement(React.Fragment, null,
                    React.createElement("div", null, "\u786E\u8BA4\u5220\u9664\u6B64\u6761\u76EE\uFF1F"),
                    React.createElement(Modal.Footer, null,
                        React.createElement(Button, { type: "primary", onClick: async () => {
                                try {
                                    modal.destroy();
                                    message.loading({ content: '删除数据中' });
                                    await datasourceExp.batchDeleteRecord([id]);
                                    navigateToModelPage('list', { dbName, id });
                                    message.success({ content: '删除数据成功' });
                                }
                                catch (error) {
                                    console.error('删除', error);
                                    alertErrorMessage({ message: `删除数据失败: ${error.message}` });
                                }
                            } }, "\u786E\u8BA4")))),
            });
        },
    };
    // 3、正常返回
    return (React.createElement(ConfigProvider, { classPrefix: "wedatea2td" },
        React.createElement(DetailLoadingContext.Provider, { value: setDetailLoading },
            React.createElement(EnumProvider, { fields: showFields },
                React.createElement(PageContent, { ...rest },
                    React.createElement(Card.Body, { style: { padding: 0, margin: 0 }, title: React.createElement("div", { className: "space-title" },
                            dbTitle,
                            "\u8BE6\u60C5"), operation: React.createElement("div", { className: "detail__operations" }, (recordData === null || recordData === void 0 ? void 0 : recordData._id) &&
                            !isProcessHoldOrLocked(recordData['!!Weda!!ProcessHold'], recordData['!!Weda!!LOCK']) &&
                            buttons &&
                            buttons.map((props, index) => (React.createElement(SelectableBlock, { ...props, component: "Button", blockKey: "buttons", compId: compId, exportValue: { record: recordData }, index: index, actionMap: actionMap, style: { marginBottom: '12px', ...props.style }, selectableBlockEvents: selectableBlockEvents })))) },
                        React.createElement(DataItemView, { dbName: dbName, fields: showFields, setRecordData: setRecordData, recordId: recordId }),
                        !detailLoading && tabChild.length > 0 && showRelatedTable && (React.createElement(React.Fragment, null,
                            React.createElement(Title, null, "\u76F8\u5173\u5217\u8868"),
                            React.createElement(Tabs, { tabs: tabs }, tabChild)))))))));
};
/**
 * Tab 子数据源内容
 */
const TabChildRender = ({ dbName, enabledFieldNames, key, childDataSourceRelatedField, detailId, compId, buttons, selectableBlockEvents, }) => {
    const [{ showFields }, setState] = useSetState({
        showFields: [],
    });
    useEffect(() => {
        async function getUseAuthFields() {
            const { showFields } = await useAuthFields(dbName, enabledFieldNames, true);
            setState({
                showFields,
            });
        }
        getUseAuthFields();
    }, [dbName]);
    const childFields = useMemo(() => showFields === null || showFields === void 0 ? void 0 : showFields.sort((m, n) => {
        if (!(m === null || m === void 0 ? void 0 : m['x-index']) || !(n === null || n === void 0 ? void 0 : n['x-index'])) {
            return 1;
        }
        return n['x-index'] - m['x-index'];
    }), [showFields]);
    const childDataSourceRelatedFieldName = childDataSourceRelatedField === null || childDataSourceRelatedField === void 0 ? void 0 : childDataSourceRelatedField[0];
    const defaultSearchValue = childDataSourceRelatedFieldName
        ? [
            {
                key: childDataSourceRelatedFieldName,
                val: detailId,
                rel: 'search',
            },
        ]
        : undefined;
    return (React.createElement("div", { key: key },
        React.createElement(EnumProvider, { fields: childFields },
            React.createElement(BaseTable, { tableAction: true, actionColumn: true, dbName: dbName, fields: childFields, defaultSearchValue: defaultSearchValue, compId: compId, buttons: buttons.map((item) => {
                    typeof item === 'object' && (item.isSelectable = false);
                    return item;
                }), selectableBlockEvents: selectableBlockEvents, onSelectConfirm: undefined, onSelectCancel: undefined, settings: undefined, filterFields: undefined, rowColor: undefined, columnSets: undefined, slots: undefined, events: undefined, isModel: true }))));
};
function getChildDataSourceRelatedField(childDatasource, parentDataSourceName) {
    var _a;
    return Object.entries(((_a = childDatasource === null || childDatasource === void 0 ? void 0 : childDatasource.schema) === null || _a === void 0 ? void 0 : _a.properties) || {}).find(([, fieldConfig]) => { var _a; return ((_a = fieldConfig === null || fieldConfig === void 0 ? void 0 : fieldConfig['x-parent']) === null || _a === void 0 ? void 0 : _a.parentDataSourceName) === parentDataSourceName; });
}
