/* eslint-disable no-param-reassign */
import React, { useState, useEffect } from 'react';
import ReactDOM from 'react-dom';
import moment from 'moment';
import { Modal, Button, LoadingTip, Status } from 'tea-component';
import { isString } from '@antv/util';
import G6 from '@antv/g6';
import { fetchProcessInfoList } from '../../services';
import { STATUS_COLOR_MAP, APPROVE_MAP } from './constants';
import './index.less';
G6.Tooltip.prototype.showTooltip = function (e) {
    if (!e.item) {
        return;
    }
    const itemTypes = this.get('itemTypes');
    if (e.item.getType && itemTypes.indexOf(e.item.getType()) === -1)
        return;
    const container = this.get('tooltip');
    const getContent = this.get('getContent');
    const tooltip = getContent(e);
    if (isString(tooltip)) {
        container.innerHTML = tooltip;
    }
    else {
        container.innerHTML = '';
        container.appendChild(tooltip);
    }
    this.updatePosition(e);
};
export const RenderProcess = React.forwardRef((props) => {
    const { commonFlowData, flowLocalProps, ide } = props;
    const { flowDetail } = commonFlowData;
    const { isWedaIde, callWedaApi, safeJsonParse } = flowLocalProps;
    const { flowCode, flowVersion: appVersion, instanceId } = flowDetail || {};
    const [visible, setVisible] = useState(false);
    const [modalData, setModalData] = useState({});
    const [loading, setLoading] = useState(false);
    const [isEmptyChart, setIsEmptyChart] = useState(false);
    useEffect(() => {
        setIsEmptyChart(!instanceId);
        instanceId && getGraphData(instanceId);
    }, [instanceId]);
    const closeModal = () => {
        setVisible(false);
    };
    const getGraphData = async (instanceId) => {
        try {
            setLoading(true);
            const params = {
                flowCode,
                instanceId,
                appVersion,
            };
            const options = { isWedaIde, callWedaApi };
            const data = await fetchProcessInfoList(params, options);
            setLoading(false);
            data && renderContent(data);
        }
        catch (ex) {
            console.log(ex);
        }
    };
    const showModal = (data, infoData) => {
        const modalData = {
            nodeName: {
                label: '审批节点',
                content: data.name,
            },
            approvePerson: {
                label: '审批人',
                content: infoData.name,
            },
            approveAction: {
                label: '审批操作',
                content: (React.createElement("span", { style: {
                        color: APPROVE_MAP[infoData.status].color,
                    } },
                    ' ',
                    APPROVE_MAP[infoData.status].txt)),
            },
            actionTime: {
                label: '操作时间',
                content: infoData.opTime ? moment(infoData.opTime).format('YYYY-MM-DD HH:mm:ss') : undefined,
            },
            approveComment: {
                label: '审批意见',
                content: infoData.comment,
            },
            newProcessors: {
                label: '操作详情',
                content: infoData.newProcessors,
            },
            attachmentInfo: {
                label: '上传附件',
                content: infoData.attachmentInfo ? safeJsonParse(infoData.attachmentInfo) : undefined,
            },
        };
        setVisible(true);
        setModalData(modalData);
    };
    G6.registerNode('sql', {
        drawShape(cfg, group) {
            const rect = group.addShape('rect', {
                attrs: {
                    x: -75,
                    y: -25,
                    width: 150,
                    height: 50,
                    radius: 10,
                    stroke: STATUS_COLOR_MAP[cfg.status].lineColor,
                    fill: STATUS_COLOR_MAP[cfg.status].bgColor,
                    lineWidth: 3,
                    lineDash: cfg.status === 3 ? [5] : undefined,
                },
                name: cfg.name,
            });
            if (cfg.name) {
                group.addShape('text', {
                    attrs: {
                        text: cfg.name,
                        x: 0,
                        y: 0,
                        fill: '#333',
                        fontSize: 12,
                        textAlign: 'center',
                        textBaseline: 'middle',
                        // fontWeight: 'bold',
                    },
                    name: 'text-shape',
                });
            }
            return rect;
        },
    }, 'single-node');
    const renderContent = (jsonData) => {
        jsonData.nodes.forEach((node) => {
            if (!node.style) {
                node.style = {};
            }
            if (node.id) {
                node.id = `${node.id}`;
            }
            node.style.lineWidth = 3;
            node.label = node.name;
            node.labelCfg = {
                fontSize: 14,
                color: '#00287E',
                fontWeight: 'bold',
            };
            // STATUS_COLOR_MAP
            node.style.stroke = STATUS_COLOR_MAP[node.status].lineColor;
            node.style.fill = STATUS_COLOR_MAP[node.status].bgColor;
            if (node.elementId.startsWith('startNode') || node.elementId.startsWith('end')) {
                node.type = 'circle';
                node.size = 80;
                if (node.status === 3) {
                    node.style.lineDash = [5];
                }
            }
            if (node.elementId.startsWith('exclusiveGateway')) {
                node.type = 'diamond';
                node.size = [160, 80];
                node.anchorPoints = [
                    [0.5, 0],
                    [0, 0.5],
                    [0.5, 1],
                    [1, 0.5],
                ];
            }
        });
        const nodeIdsMap = jsonData.nodes.reduce((acc, cur) => {
            acc[cur.elementId] = cur.id;
            return acc;
        }, {});
        jsonData.edges.forEach((edge) => {
            edge.source = nodeIdsMap[edge.source];
            edge.target = nodeIdsMap[edge.target];
            edge.labelCfg = {
                refY: -20,
            };
            Reflect.deleteProperty(edge, 'type');
            if (edge.status === 1) {
                edge.style = {
                    stroke: '#0ABF5B',
                };
            }
        });
        const container = document.getElementById('flow-graph_container');
        const width = container.scrollWidth;
        const height = document.documentElement.clientHeight;
        const tooltip = new G6.Tooltip({
            offsetX: 60,
            offsetY: -20,
            trigger: 'click',
            getContent(e) {
                const outDiv = document.createElement('div');
                outDiv.style.width = 'auto';
                const modelData = e.item.getModel();
                const conf = modelData === null || modelData === void 0 ? void 0 : modelData.conf;
                let vdomContent;
                if (conf === null || conf === void 0 ? void 0 : conf.length) {
                    const contentHtml = conf.map((value, index) => {
                        const info = { ...value };
                        // 区分处理节点和审批节点的 待审批/待处理 文案
                        if (modelData.dataType === 23 && info.status === 0) {
                            info.status = 10;
                        }
                        return (React.createElement("li", { className: "ProcessInfo-item", key: `conf-${index}`, onClick: () => {
                                showModal(modelData, info);
                            } },
                            React.createElement("div", { className: "first-line" },
                                React.createElement("i", { className: info.status ? 'tootip-will_do' : '' }),
                                React.createElement("span", { className: "tootip-name" }, info.name),
                                React.createElement("span", { style: {
                                        color: APPROVE_MAP[info.status ? info.status : 0].color,
                                        paddingLeft: '8px',
                                    } }, APPROVE_MAP[info.status ? info.status : 0].txt)),
                            info.status && info.opTime ? (React.createElement("p", { className: "tooltip-time" }, moment(info.opTime).format('YYYY-MM-DD HH:mm:ss'))) : null));
                    });
                    vdomContent = (React.createElement("div", { className: "ProcessInfo" },
                        React.createElement("h4", null, "\u8282\u70B9\u5BA1\u6279\u4FE1\u606F"),
                        React.createElement("ul", { className: "ProcessInfo-content" }, contentHtml)));
                }
                else {
                    vdomContent = React.createElement("p", null, "\u6682\u65E0\u4FE1\u606F");
                }
                ReactDOM.render(vdomContent, outDiv);
                return outDiv;
            },
            itemTypes: ['node'],
        });
        const graph = new G6.Graph({
            container: 'flow-graph_container',
            width,
            height: height - 300,
            fitView: true,
            fitViewPadding: [20, 40, 50, 20],
            layout: {
                type: 'dagre',
                ranksep: 30,
                controlPoints: true,
            },
            defaultNode: {
                type: 'sql',
            },
            plugins: [tooltip],
            defaultEdge: {
                type: 'polyline',
                style: {
                    endArrow: true,
                    lineWidth: 2,
                    stroke: '#C2C8D5',
                },
            },
            nodeStateStyles: {
                selected: {
                    stroke: '#000',
                    fill: '#5394ef',
                },
            },
            modes: {
                default: ['drag-canvas', 'zoom-canvas'],
            },
        });
        graph.data(jsonData);
        graph.render();
        if (typeof window !== 'undefined')
            window.onresize = () => {
                if (!graph || graph.get('destroyed'))
                    return;
                if (!container || !container.scrollWidth || !container.scrollHeight)
                    return;
                graph.changeSize(container.scrollWidth, container.scrollHeight);
            };
    };
    const renderStatus = () => {
        if (loading) {
            return React.createElement(LoadingTip, null);
        }
        if (isEmptyChart) {
            return React.createElement(Status, { icon: "blank", size: "l", title: "\u6682\u65E0\u6D41\u8F6C\u8282\u70B9" });
        }
        return null;
    };
    return (React.createElement("div", { id: "flow-graph_container" },
        renderStatus(),
        React.createElement(Modal, { visible: visible, caption: '审批详情', onClose: closeModal, className: "ProcessInfo-modal" },
            React.createElement(Modal.Body, null,
                React.createElement("div", { className: "ProcessInfo-modal_content" }, Object.entries(modalData)
                    .filter(([, { content }]) => content)
                    .map(([, { label, content }]) => (React.createElement("div", null,
                    React.createElement("span", { className: "ProcessInfo-modal_content-label" }, label),
                    React.createElement("span", { className: "ProcessInfo-modal_content-content" }, content)))))),
            React.createElement(Modal.Footer, null,
                React.createElement(Button, { type: "weak", onClick: closeModal }, "\u5173\u95ED")))));
});
