/* eslint-disable no-param-reassign */
import React, { useState, useRef, useEffect, useCallback, createContext, useMemo, useLayoutEffect, } from 'react';
import { StatusTip } from 'tea-component';
// 流程图绘制
import ReactFlow, { addEdge, isNode, Controls, Position } from 'react-flow-renderer';
// 流程图布局
import dagre from 'dagre';
// 缩放按钮
import FlowControl from './components/FlowControl';
// 自定义流程元素
import { TriggerNode, EndNode, PlusEdge, SingleNode, ApprovalNode, EditDataTableNode, SendMsgNode, BlankNode, GatewayNode, } from './components';
// 引入流程自定义常量
import { FLOW_LAYOUT_DIRECTION, FLOW_LAYOUT_NODE_SEP, FLOW_LAYOUT_NODE_WIDTH, FLOW_LAYOUT_NODE_HEIGHT, FLOW_BACKGROUND_COLOR, FLOW_ZOOM_DEFAULT, FLOW_ZOOM_MIN, FLOW_ZOOM_MAX, FLOW_SINGLE_NODE, FLOW_LAYOUT_NODE_WIDTH_DEFAULT, FLOW_LAYOUT_NODE_HEIGHT_DEFAULT, FLOW_DEFAULT_DATA, FLOW_BPMN_TO_ELEMENT_TYPE, FLOW_BPMN_EXCLUSIVE_GATEWAY_NODE, FLOW_BLANK_NODE, FLOW_GATEWAY_NODE, } from './constants';
import { useRequest } from 'ahooks';
import { initData } from './utils';
import { PageContent } from '../PageContent';
import { describeProcessInstanceNodes } from '../../services';
import { useFlowContext } from '../FlowCard/Context';
import './index.less';
// 组件共享上下文
export const FlowContext = createContext(null);
// 引入自定义连线
const edgeTypes = {
    plus: PlusEdge,
};
// 编辑器组件
export const FlowChart = (props) => {
    const { processname, instanceid } = props;
    const [flowData, setFlowData] = useState(FLOW_DEFAULT_DATA);
    const [elements, setElements] = useState();
    const [show, setShow] = useState(false);
    const flowRef = useRef();
    const { callWorkflow, IS_WEDA_IDE, showToast } = useFlowContext();
    // 获取流程详情请求
    const { run: runFlowDetails, loading } = useRequest(describeProcessInstanceNodes, {
        manual: true,
    });
    // 引入自定义节点
    const nodeTypes = {
        triggerNode: TriggerNode,
        endNode: EndNode,
        singleNode: SingleNode,
        approvalNode: ApprovalNode,
        editDataTableNode: EditDataTableNode,
        sendMsgNode: SendMsgNode,
        blankNode: BlankNode,
        gatewayNode: GatewayNode,
    };
    // 添加连线
    // @ts-ignore
    const onConnect = (params) => setElements((els) => addEdge(params, els));
    // 响应体转流程数据
    const responseTransform = useCallback((responseData) => {
        const modelData = [];
        const { ProcessNodeList, SequenceFlowList } = responseData;
        if (ProcessNodeList && ProcessNodeList.length > 0) {
            ProcessNodeList.forEach((node) => {
                const { NodeKey, NodeName, ElementType, TriggerNode, ManualUserNode, NoticeNode, AutoServiceNode, BranchEnd, Status, } = node;
                if (NodeKey && ElementType && NodeKey !== 'Node-0') {
                    if (!NodeKey.includes('Gateway-')) {
                        const NodeId = NodeKey.replace('Node-', '');
                        const commonParams = {
                            id: NodeId,
                            status: Status,
                            branchEnd: BranchEnd || '',
                            type: FLOW_BPMN_TO_ELEMENT_TYPE[ElementType],
                        };
                        if (NoticeNode) {
                            modelData.push({
                                ...commonParams,
                                NoticeNode,
                            });
                        }
                        else if (AutoServiceNode) {
                            modelData.push({
                                ...commonParams,
                                AutoServiceNode,
                            });
                        }
                        else if (TriggerNode) {
                            modelData.push({
                                ...commonParams,
                                TriggerNode,
                            });
                        }
                        else if (ManualUserNode) {
                            modelData.push({
                                ...commonParams,
                                ManualUserNode,
                            });
                        }
                        else {
                            modelData.push(commonParams);
                        }
                    }
                    else {
                        const ids = NodeName.replace(`${FLOW_BPMN_EXCLUSIVE_GATEWAY_NODE}-`, '').split('-');
                        modelData.push({
                            id: NodeKey.replace('Gateway-', ''),
                            type: FLOW_BPMN_TO_ELEMENT_TYPE[ElementType],
                            status: Status,
                            branchEnd: BranchEnd || '',
                        });
                        ids.forEach((i) => {
                            modelData.push({
                                id: i,
                                type: FLOW_SINGLE_NODE,
                                branchEnd: '',
                            });
                        });
                    }
                }
            });
        }
        if (SequenceFlowList && SequenceFlowList.length > 0) {
            SequenceFlowList.forEach((sequence) => {
                const { FlowKey, SourceNodeKey, TargetNodeKey, DatasourceExtension = {} } = sequence;
                if (FlowKey && SourceNodeKey && TargetNodeKey && SourceNodeKey !== 'Node-0') {
                    if (SourceNodeKey.includes('Gateway-')) {
                        const FlowId = FlowKey.replace('Node-', '');
                        const middleId = FlowId.split('-')[0];
                        const sourceId = SourceNodeKey.replace('Gateway-', '');
                        const targetId = TargetNodeKey.replace('Node-', '').replace('Gateway-', '');
                        modelData.push({ id: `${sourceId}-${middleId}`, source: sourceId, target: middleId });
                        modelData.push({ id: FlowId, source: middleId, target: targetId });
                        const singleNode = modelData[modelData.findIndex((item) => item.id === middleId)];
                        singleNode.ConditionShowName = DatasourceExtension.ConditionShowName;
                        const nodeOfTarget = modelData[modelData.findIndex((item) => item.id === targetId)] || {};
                        singleNode.status = nodeOfTarget.status || '';
                    }
                    else {
                        const FlowId = FlowKey.replace('Node-', '');
                        const sourceId = SourceNodeKey.replace('Node-', '').replace('Gateway-', '');
                        const targetId = TargetNodeKey.replace('Node-', '').replace('Gateway-', '');
                        modelData.push({ id: FlowId, source: sourceId, target: targetId });
                    }
                }
            });
        }
        return modelData;
    }, []);
    // 从接口获取数据
    useEffect(() => {
        async function getFlowDetails() {
            const flowDetails = await runFlowDetails({
                processInstanceId: instanceid,
            }, IS_WEDA_IDE, callWorkflow, showToast);
            const flowJsonToData = flowDetails ? responseTransform(flowDetails) : FLOW_DEFAULT_DATA;
            setFlowData(Object.assign([], flowJsonToData));
            setShow(true);
        }
        getFlowDetails();
    }, [runFlowDetails, responseTransform]);
    // 格式化流程图布局
    useLayoutEffect(() => {
        var _a;
        const dagreGraph = new dagre.graphlib.Graph({ directed: true });
        // 覆盖连线的默认节点文本为空
        dagreGraph.setDefaultEdgeLabel(() => ({}));
        // nodesep表示节点间距
        dagreGraph.setGraph({
            rankdir: FLOW_LAYOUT_DIRECTION,
            nodesep: FLOW_LAYOUT_NODE_SEP,
        });
        const initElements = initData(flowData);
        initElements.forEach((el) => {
            if (isNode(el)) {
                if (initElements.filter((dataItem) => isNode(dataItem)).length <= 2) {
                    dagreGraph.setNode(el.id, {
                        height: FLOW_LAYOUT_NODE_HEIGHT_DEFAULT,
                        width: FLOW_LAYOUT_NODE_WIDTH_DEFAULT,
                    });
                }
                else {
                    dagreGraph.setNode(el.id, {
                        height: el.type === FLOW_BLANK_NODE || el.type === FLOW_GATEWAY_NODE
                            ? 0
                            : FLOW_LAYOUT_NODE_HEIGHT,
                        width: el.type === FLOW_BLANK_NODE || el.type === FLOW_GATEWAY_NODE
                            ? 0
                            : FLOW_LAYOUT_NODE_WIDTH,
                    });
                }
            }
            else {
                dagreGraph.setEdge(el.source, el.target, {
                    minlen: 1, // 手动控制节点的位置
                });
            }
        });
        dagre.layout(dagreGraph);
        // 图表区一半
        // @ts-ignore
        const halfClientWidth = ((_a = flowRef.current) === null || _a === void 0 ? void 0 : _a.offsetWidth) / 2;
        // 第一个节点的x坐标
        const firstNodeX = dagreGraph.node('1').x;
        // 需要移动到中间的距离，150是节点本身的一半
        const gap = halfClientWidth - firstNodeX - 150;
        const gapMap = {
            [FLOW_GATEWAY_NODE]: 60,
            [FLOW_BLANK_NODE]: 40,
        };
        const layoutedElements = initElements.map((el) => {
            if (isNode(el)) {
                const nodeWithPosition = dagreGraph.node(el.id);
                el.targetPosition = Position.Top;
                el.sourcePosition = Position.Bottom;
                // 如果需要移动，则移到中间位置
                el.position = {
                    x: nodeWithPosition.x + (gap > 0 ? gap : 0),
                    y: el.type === FLOW_GATEWAY_NODE || el.type === FLOW_BLANK_NODE
                        ? nodeWithPosition.y + gapMap[el.type]
                        : nodeWithPosition.y,
                };
            }
            return el;
        });
        setElements(layoutedElements);
    }, [flowData]);
    // 修改cursor
    const changeCursor = (cursor) => {
        const flowDom = flowRef.current;
        if (flowDom) {
            // @ts-ignore
            flowDom.style.setProperty('cursor', cursor, 'important');
        }
    };
    const providerValue = useMemo(() => ({ flowData, elements }), [flowData, elements]);
    return (React.createElement(PageContent, { ...props },
        !show && (React.createElement("div", { style: { textAlign: 'center', width: '100%', paddingTop: '15%' } },
            React.createElement(StatusTip, { status: loading ? 'loading' : 'empty' }))),
        React.createElement(FlowContext.Provider, { value: providerValue }, elements && (React.createElement(ReactFlow, { ref: flowRef, elements: elements, onLoad: ({ fitView }) => {
                fitView();
            }, onConnect: onConnect, style: { background: FLOW_BACKGROUND_COLOR, minHeight: '400px' }, className: show ? 'flow_editor_show' : 'flow_editor_hidden', nodeTypes: nodeTypes, edgeTypes: edgeTypes, nodesDraggable: false, nodesConnectable: false, elementsSelectable: false, zoomOnDoubleClick: false, selectNodesOnDrag: false, defaultZoom: FLOW_ZOOM_DEFAULT, minZoom: FLOW_ZOOM_MIN, maxZoom: FLOW_ZOOM_MAX, onClick: () => {
                changeCursor('grab');
            }, onMove: () => {
                changeCursor('grabbing');
            }, onMoveEnd: () => {
                changeCursor('grab');
            } },
            React.createElement(Controls
            // 关闭默认按钮，使用自定义组件代替
            , { 
                // 关闭默认按钮，使用自定义组件代替
                showZoom: false, showInteractive: false, showFitView: false, style: { display: 'flex', bottom: '16px', left: '16px' } },
                React.createElement(FlowControl, { flowRef: flowRef, flowName: processname })))))));
};
