import React, { useState, useEffect } from 'react';
import { ConfigProvider } from 'tea-component';
import { PageContent, PageNullDb } from '../PageContent';
import { BaseTable } from './BaseTable';
import { useAuthFields } from '../../hooks';
import { TABLE_BUTTONS, TABLE_SETTINGS, TABLE_SLOT_PREFIX } from '../../constants';
import { EnumProvider } from '../../context';
import { alertErrorMessage } from '../../utils/feedback';
/**
 * 列表组件
 */
export const ModelTable = React.forwardRef((props, ref) => {
    const { dataSourceType, bindConnectMetadata = {
        datasource: { id: '', name: '', title: '' },
    }, // APImock数据
    connectorMethod = { name: '', inParams: [], outParams: [], title: '' }, // API方法
    connectorParams, // API查询入参
    isOrderBy, // API是否支持排序
    bindMetadata = { datasource: {}, enabledFieldNames: [], selectedViewIds: [] }, // 数据源+字段data对象
    filterType = 'searchbox', // 条件筛选类型
    filterFields, // 筛选字段
    buttons = TABLE_BUTTONS, // 选区按钮集合
    settings = TABLE_SETTINGS, // 列表操作列默认按钮选区
    _selectableBlockEvents: selectableBlockEvents, // 选区的事件集合
    columnSets, // 列管理，具备列排序/列勾选/列自定义渲染作用，存在时代替enabledFieldNames
    rowColor, // 行背景色
    id: compId, // 组件id
    events, // 事件
    where, // 搜索条件
    orderBy, // 排序字段
    orderType, // 排序方式
    fixedFront = '', // 冻结首列至本列
    fixedEnd = '', // 冻结本列至尾列
    ...rest } = props;
    const isModel = dataSourceType !== 'custom-connector';
    const { datasource, enabledFieldNames, selectedViewIds: selectedView, name, isNewData, } = isModel ? bindMetadata : bindConnectMetadata;
    const { inParams, outParams, title } = connectorMethod;
    const dbName = datasource === null || datasource === void 0 ? void 0 : datasource.name;
    const dbFieldNames = isModel ? enabledFieldNames : outParams;
    const [showFields, setShowFields] = useState([]);
    const [getFieldsLoading, setGetFieldsLoading] = useState(true);
    useEffect(() => {
        try {
            async function getShowFields() {
                const { showFields: value } = await useAuthFields(dbName, dbFieldNames, true, !!isNewData, isModel, connectorMethod);
                setShowFields(value);
                setGetFieldsLoading(false);
            }
            getShowFields();
        }
        catch (err) {
            setGetFieldsLoading(false);
            setShowFields([]);
            alertErrorMessage({ message: `获取数据源字段失败,错误信息:${err}` });
            console.error(`获取数据源字段失败,错误信息:${err}`);
        }
    }, [dbName]);
    /**
     * 通过前缀过滤出插槽属性，传入属性是确定的，所以可以通过前缀识别
     */
    const slots = Object.keys(rest).reduce((acc, key) => {
        if (key.startsWith(TABLE_SLOT_PREFIX)) {
            acc[key] = rest[key];
        }
        return acc;
    }, {});
    /**
     * 获取选中的筛选字段
     */
    const getFilterFields = (filterFields) => {
        var _a;
        let result = [];
        try {
            const filterFieldsObj = JSON.parse(JSON.stringify(filterFields));
            result = ((_a = filterFieldsObj === null || filterFieldsObj === void 0 ? void 0 : filterFieldsObj.selected) === null || _a === void 0 ? void 0 : _a.map((i) => showFields.find((j) => j.name === i))) || [];
        }
        catch (error) {
            result = [];
        }
        return result;
    };
    /**
     * 处理选择视图的意外情况(存量用户不会更新数据源的默认视图可能会出现[null])
     */
    const getSelectedView = (selectedViews) => {
        if (selectedViews && (selectedViews === null || selectedViews === void 0 ? void 0 : selectedViews.length) > 0) {
            return selectedViews.filter((item) => !!item);
        }
        return [];
    };
    // 1、编辑器内未绑定数据源时返回
    if ((isModel && !dbName) || (!isModel && !(connectorMethod === null || connectorMethod === void 0 ? void 0 : connectorMethod.name))) {
        return React.createElement(PageNullDb, { ...rest });
    }
    // 2、正常返回
    return (React.createElement(ConfigProvider, { classPrefix: "wedatea2td" },
        React.createElement(EnumProvider, { fields: showFields },
            React.createElement(PageContent, { ...rest, loading: getFieldsLoading },
                React.createElement(BaseTable, { filterType: filterType, filterFields: getFilterFields(filterFields), actionColumn: true, tableAction: true, dbName: dbName, fields: showFields, connectorParams: connectorParams, isOrderBy: isOrderBy, isModel: isModel, tableSearch: !!isModel, connectorMethod: connectorMethod === null || connectorMethod === void 0 ? void 0 : connectorMethod.name, compId: compId, buttons: buttons, settings: settings, selectableBlockEvents: selectableBlockEvents, selectedView: getSelectedView(selectedView), columnSets: columnSets, slots: slots, rowColor: rowColor, events: events, onSelectConfirm: undefined, onSelectCancel: undefined, defaultSearchValue: undefined, outerRef: ref, where: where, orderBy: orderBy, orderType: orderType, fixedFront: fixedFront, fixedEnd: fixedEnd })))));
});
export { BaseTable };
