import React, { useEffect } from 'react';
import { useRequest, useSetState } from 'ahooks';
import { message } from 'tea-component';
import { ModelForm } from '../Form';
import { PageContent, PageCountDown, PageNullDb } from '../PageContent';
import { DataSource, getPageQuery, navigateToModelPage, isInIde, isEmpty } from '../../utils';
import { useAuthFields } from '../../hooks';
import { UPDATE_BUTTONS } from '../../constants/';
import { alertErrorMessage } from '../../utils/feedback';
/**
 * 更新组件
 */
export const ModelUpdate = (props) => {
    const { bindMetadata = {}, // data属性
    buttons = UPDATE_BUTTONS, // 选区按钮集合
    _selectableBlockEvents: selectableBlockEvents, // 选区的事件集合
    id: compId, // 组件的id
    ...rest } = props;
    const { datasource, enabledFieldNames } = bindMetadata;
    const { id } = getPageQuery();
    const dbName = datasource === null || datasource === void 0 ? void 0 : datasource.name;
    const [{ showFields, requireNoRwFields, authLoading }, setState] = useSetState({
        showFields: [],
        requireNoRwFields: [],
        authLoading: false,
    });
    useEffect(() => {
        async function getUseAuthFields() {
            const { showFields, requireNoRwFields, authLoading } = await useAuthFields(dbName, enabledFieldNames);
            setState({
                showFields,
                requireNoRwFields,
                authLoading,
            });
        }
        getUseAuthFields();
    }, [dbName]);
    const { data = {}, loading } = useRequest(async () => {
        if (!id && !isInIde()) {
            alertErrorMessage({ message: '记录 ID 不存在，加载数据失败' });
            return {};
        }
        const db = new DataSource(dbName);
        const data = await db.getRecord(id);
        if (isEmpty(data) && !isInIde()) {
            alertErrorMessage({ message: '记录 ID 不存在，加载数据失败' });
            return {};
        }
        return data;
    });
    // 1、编辑器内未绑定数据源时返回
    if (!dbName) {
        return React.createElement(PageNullDb, { ...rest });
    }
    // 2、未找到url上的数据源id时返回
    if (!id && !isInIde()) {
        alertErrorMessage({ message: '数据记录 ID 不存在，加载数据失败' });
        return React.createElement(PageCountDown, { dbName: dbName, description: "\u8BB0\u5F55 ID \u4E0D\u5B58\u5728", ...rest });
    }
    // 3、无编辑权限字段时返回
    if (showFields.length === 0 && !authLoading) {
        if (isInIde()) {
            return React.createElement(PageContent, { ...rest }, "\u672A\u627E\u5230\u5B57\u6BB5\u4FE1\u606F");
        }
        else {
            return React.createElement(PageCountDown, { dbName: dbName, ...rest });
        }
    }
    // 4、正常返回
    return (React.createElement(PageContent, { loading: loading || (authLoading && !isInIde()), ...rest },
        React.createElement(ModelForm, { action: "update", initialData: data, fields: showFields, compId: compId, buttons: buttons, selectableBlockEvents: selectableBlockEvents, onSubmit: async (v) => {
                try {
                    if (requireNoRwFields.length > 0) {
                        message.warning({ content: '当前有不具备权限的字段是必填项，无法进行保存' });
                        return;
                    }
                    const subValue = {};
                    // 提交时只提交有读写权限的字段
                    Object.keys(v).forEach((item) => {
                        const value = showFields === null || showFields === void 0 ? void 0 : showFields.find((k) => k.name === item);
                        if (value && (value === null || value === void 0 ? void 0 : value.readWriteAuth) === 'rw') {
                            subValue[item] = v[item];
                        }
                    });
                    const db = new DataSource(dbName);
                    await db.updateRecord(id, subValue);
                    message.success({ content: '更新数据成功' });
                    navigateToModelPage('list', { dbName });
                }
                catch (error) {
                    alertErrorMessage({ message: `更新数据失败 ${error.message}` });
                }
            } })));
};
