/**
 * 流程组件请求api
 * 公共参数
 * @param  {[String]} Action [接口动作]
 * @param  {[String]} Version [本接口取值：2021-01-08。]
 */
import { flowData } from '../mock';
const NO_CALLWORKFLOW_TIPS = {
    title: '没有callWorkflow方法',
    icon: 'error',
};
const NO_CALLWEDAAPI_TIPS = {
    title: '没有callWedaApi方法',
    icon: 'error',
};
/**
 * 根据数据源id查询流程实例信息
 * https://tcloud-dev.oa.com/document/product/1505/58830?!preview&!document=1
 * {[String]} DatasourceId [数据源Id]
 * {[String]} ItemId [数据源记录Id]
 * {[ViewId]} DatasourceId [视图Id]
 * {[EnvType]} EnvType [预览：pre；非预览：prod]
 * {[ProcessKey]} ProcessKey [流程定义Key]
 */
export async function getProcessInstanceInfoByItemId({ processInstanceId, itemId, datasourceName, processKey, viewId, callWorkflow, showToast, showError, IS_WEDA_IDE, }) {
    if (IS_WEDA_IDE)
        return Promise.resolve(flowData);
    if (callWorkflow) {
        try {
            const res = await callWorkflow({
                action: 'DescribeApproveComponentNodes',
                data: {
                    ProcessInstanceId: processInstanceId,
                    ItemId: itemId,
                    DatasourceName: datasourceName,
                    IncludeApprovalNode: true,
                    ProcessKey: processKey,
                    ViewId: viewId,
                },
            });
            return Promise.resolve(res);
        }
        catch (error) {
            showError(error);
        }
    }
    else {
        showToast(NO_CALLWORKFLOW_TIPS);
        Promise.reject(NO_CALLWORKFLOW_TIPS.title);
    }
}
/**
 * 审批通过
 * https://tcloud-dev.oa.com/document/product/1505/58791?!preview&!document=1
 * {[String]} TaskId [流程标识]
 * {[String]} EnvType [环境信息， pre: 预览环境，pro: 正式环境，默认为预览环境。]
 * {[Array of String	]} Variables.N	 [流程中的变量设置]
 */
export function getApprovedTask(data, callWorkflow, showToast) {
    if (!callWorkflow) {
        showToast(NO_CALLWORKFLOW_TIPS);
        return Promise.reject(NO_CALLWORKFLOW_TIPS.title);
    }
    return callWorkflow({
        action: 'ApproveTask',
        data,
    });
}
/**
 * 审批驳回
 * https://tcloud-dev.oa.com/document/product/1505/58791?!preview&!document=1
 * {[String]} TaskId [流程标识]
 * {[String]} EnvType [环境信息， pre: 预览环境，pro: 正式环境，默认为预览环境。]
 * {[String	]} NodeId	 [驳回到流程中的某个节点 Id 标识]
 * {[Array of String	]} Variables.N	 [流程中的变量设置]
 */
export function getRejectedTask(data, callWorkflow, showToast) {
    if (!callWorkflow) {
        showToast(NO_CALLWORKFLOW_TIPS);
        return Promise.reject(NO_CALLWORKFLOW_TIPS.title);
    }
    return callWorkflow({
        action: 'RejectTask',
        data,
    });
}
/**
 * 审批流程撤回
 * https://tcloud-dev.oa.com/document/product/1505/58791?!preview&!document=1
 * {[String]} TaskId [流程标识]
 * {[String]} EnvType [环境信息， pre: 预览环境，pro: 正式环境，默认为预览环境。]
 * {[String	]} ProcessInstanceId	 [流程实例Id]
 * {[Array of String	]} Variables.N	 [流程中的变量设置]
 */
export function getRevokeTask(data, callWorkflow, showToast) {
    if (!callWorkflow) {
        showToast(NO_CALLWORKFLOW_TIPS);
        return Promise.reject(NO_CALLWORKFLOW_TIPS.title);
    }
    return callWorkflow({
        action: 'RevokeTask',
        data,
    });
}
/*
  获取审批人 用户列表  https://tcloud-dev.oa.com/document/product/1505/57752?!preview&!document=1
*/
export async function getApproversList(data, callWedaApi, showError, showToast) {
    if (callWedaApi) {
        try {
            const res = await callWedaApi({
                action: 'DescribeWedaUserList',
                data: { ...data, UserType: 'internalUser' },
            });
            return Promise.resolve(res);
        }
        catch (error) {
            showError(error);
        }
    }
    else {
        showToast(NO_CALLWEDAAPI_TIPS);
        Promise.reject(NO_CALLWEDAAPI_TIPS.title);
    }
}
/**
 * 根据名称查询审批人 https://tcloud-dev.oa.com/document/product/1505/59233?!preview&!document=1
 */
export async function getApproversListByName(name, callWedaApi, showError, showToast) {
    if (callWedaApi) {
        try {
            const res = await callWedaApi({
                action: 'DescribeUserServerByName',
                data: {
                    Name: name,
                },
            });
            return Promise.resolve(res);
        }
        catch (error) {
            showError(error);
        }
    }
    else {
        showToast(NO_CALLWEDAAPI_TIPS);
        Promise.reject(NO_CALLWEDAAPI_TIPS.title);
    }
}
/*
 * 获取下一个审批节点信息 https://tcloud-dev.oa.com/document/product/1505/59406?!preview&!document=1
 */
export async function getNextApprovalNodeInfo(ProcessInstanceId, NodeId, Variables, AddSignTaskType, callWorkflow, showError, showToast) {
    if (callWorkflow) {
        try {
            const res = await callWorkflow({
                action: 'DescribeNextProcessNode',
                data: {
                    ProcessInstanceId,
                    NodeId,
                    Variables,
                    AddSignTaskType,
                },
                capitalizeDataKey: false,
            });
            return Promise.resolve(res);
        }
        catch (error) {
            showError(error);
        }
    }
    else {
        showToast(NO_CALLWORKFLOW_TIPS);
        Promise.reject(NO_CALLWORKFLOW_TIPS.title);
    }
}
