import jsonExport from 'jsonexport/dist';
import { createRandomString } from './tool';
import moment from 'moment';
import { utils, writeFile, SSF, readFile } from 'xlsx';
/**
 * 将数据导出为 CSV 或 JSON
 */
export const exportData = async (data, fileType = 'csv', name = 'weda-data-export') => {
    const tail = moment().format('YYYYMMDDHHmmss') || createRandomString(8);
    const title = `${name}-${tail}.${fileType}`;
    try {
        if (fileType === 'json') {
            await saveContentToFile(JSON.stringify(data), title);
        }
        else if (fileType === 'xls' || fileType === 'xlsx') {
            const book = utils.book_new();
            const sheet = utils.aoa_to_sheet(data);
            const reversalData = data[0].map((col, i) => data.map((row) => row[i]));
            sheet['!cols'] = reversalData.map((cells) => ({ wch: getColumnWidth(cells) }));
            utils.book_append_sheet(book, sheet, 'Sheet1');
            writeFile(book, title);
        }
        else {
            let csv = await jsonExport(data);
            csv = `\ufeff${csv}`;
            await saveFile(new Blob([csv], { type: 'text/csv,charset=utf-8' }), title);
        }
    }
    catch (e) { }
};
/**
 * 取每列最大宽度，实现excel列宽自适应
 */
const getColumnWidth = (cells = []) => {
    const widths = cells === null || cells === void 0 ? void 0 : cells.map((d) => {
        if (!d)
            return 10;
        const value = `${d}`;
        return value.replace(/[\u0391-\uFFE5]/g, 'aa').length * 1.1;
    });
    return Math.max(...widths);
};
/**
 * 将内容保存为文件
 */
export const saveContentToFile = (content, fileName, type = 'application/json') => {
    if (!/\.json$/.test(fileName) && type === 'application/json') {
        throw new Error('文件格式指定错误');
    }
    return saveFile(new Blob([content], { type }), fileName);
};
/**
 * 浏览器保存文件到本地
 * @param {Blob} file
 * @param {string} fileName
 */
export const saveFile = (file, fileName) => {
    // @ts-ignore
    if (window.navigator.msSaveOrOpenBlob) {
        // IE10+
        // @ts-ignore
        window.navigator.msSaveOrOpenBlob(file, fileName);
    }
    else {
        const url = URL.createObjectURL(file);
        downloadFileFromUrl(url, fileName);
    }
};
/**
 * 通过链接下载文件
 */
export const downloadFileFromUrl = (url, fileName) => {
    // 创建一个链接
    const a = document.createElement('a');
    a.href = url;
    a.download = fileName;
    document.body.appendChild(a);
    a.click();
    setTimeout(() => {
        document.body.removeChild(a);
        window.URL.revokeObjectURL(url);
    }, 0);
};
/**
 * 读取上传的xlsx文件
 */
export const getXlsxFileToJson = (file, cb) => {
    const reader = new FileReader();
    reader.readAsArrayBuffer(file);
    reader.onloadend = () => {
        const wb = readFile(reader.result, { type: 'binary', cellDates: true });
        const result = utils.sheet_to_json(wb.Sheets[wb.SheetNames[0]], {
            header: 1,
            cellDates: true,
        });
        const [header, ...rest] = result;
        const headers = Array.isArray(header) ? header : [];
        // 修正时间格式的日期读取结果
        const fixed = fixSheetToJson(rest, wb);
        const xlsxToJsonResult = [];
        fixed.forEach((i) => {
            const temp = {};
            i.forEach((n, index) => {
                temp[headers[index]] = n;
            });
            xlsxToJsonResult.push(temp);
        });
        if (cb) {
            cb({ xlsxToJsonResult, headers });
        }
    };
};
/**
 * 读取上传的xlsx文件的表头
 */
export const getXlsxFileHeader = (file, cb) => {
    const reader = new FileReader();
    reader.readAsArrayBuffer(file);
    reader.onloadend = () => {
        const wb = readFile(reader.result, {
            type: 'binary',
            cellDates: true,
        });
        const result = utils.sheet_to_json(wb.Sheets[wb.SheetNames[0]], {
            header: 1,
            cellDates: true,
        });
        const [header] = result;
        const headers = Array.isArray(header) ? header : [];
        if (cb) {
            cb({ headers });
        }
    };
};
const datenum = (v, date1904) => {
    /** excel 的时间从 1900/1/0（1899/12/30）开始 ，此时对中国用的上海时区（GMT+0805）+8:05:43，1800～1900年。
        时区是动态的，根据时间的变化而变化
        getTimezoneOffset 获取的分钟数，忽略了43秒，所以会少 43 秒
        https://github.com/SheetJS/sheetjs/issues/1565
        */
    const basedate = new Date(1899, 11, 30, 0, 0, 0);
    // 重新计算时间，精确到秒
    const dnthresh = basedate.getTime() + (new Date().getTimezoneOffset() - basedate.getTimezoneOffset()) * 60000;
    const dayMs = 24 * 60 * 60 * 1000;
    const days1462Ms = 1462 * dayMs;
    let epoch = v.getTime();
    if (date1904) {
        epoch -= days1462Ms;
    }
    return (epoch - dnthresh) / dayMs;
};
const fixImportedDate = (date, isDate1904) => {
    /** ssf是一个处理日期的js库，有两种模式处理时间
        1900 mode (where date=0 is 1899-12-31)
        1904 mode (where date=0 is 1904-01-01)
       */
    // 利用ssf库解析时间，获得最终时间
    const parsed = SSF.parse_date_code(datenum(date, false), { date1904: isDate1904 });
    return new Date(parsed.y, parsed.m, parsed.d, parsed.H, parsed.M, parsed.S);
};
const fixSheetToJson = (rest, wb) => {
    // apply hotfix
    const isDate1904 = wb.Workbook.WBProps.date1904;
    const fixed = rest.map((arr) => arr.map((v) => {
        if (v instanceof Date) {
            return fixImportedDate(v, isDate1904);
        }
        return v;
    }));
    return fixed;
};
