import { getDataSourceByName, getDataSourceFieldsWithAuth } from './datasource';
// 内置字段
const SystemDefaultFields = ['_id', 'createdAt', 'updatedAt', 'owner', 'createBy', 'updateBy'];
/**
 * 将 schema 定义中的 properties 转换为 fields
 */
export const resolveFieldsFromSchema = (schema) => {
    if (!(schema === null || schema === void 0 ? void 0 : schema.properties))
        return [];
    const keys = Object.keys(schema.properties).filter((_) => _ !== '_id');
    return keys.map((key) => ({
        ...schema.properties[key],
        name: key,
    }));
};
/**
 * 判断一个字段是否为内置系统字段
 */
export const isSystemField = (field) => {
    if ((field === null || field === void 0 ? void 0 : field['x-system']) === undefined) {
        return SystemDefaultFields.includes(field === null || field === void 0 ? void 0 : field.name);
    }
    return (field === null || field === void 0 ? void 0 : field['x-system']) === true;
};
/**
 * 根据数据库名，解析 fields
 * isUseAll：表格组件不使用enabledFieldNames，而是使用columnSets
 */
export const resolveFieldsFromDatasource = async ({ dbName, enabledFieldNames, displaySystem, isUseAll, isModel = true, connectorMethod, }) => {
    var _a, _b, _c, _d;
    if (isModel) {
        if ((!dbName || !(enabledFieldNames === null || enabledFieldNames === void 0 ? void 0 : enabledFieldNames.length)) && !isUseAll)
            return [];
        const datasource = await getDataSourceByName(dbName, true);
        // 警告
        if (!datasource) {
            console.error(`未找到数据源 ${dbName}`, window.app.cloud);
            return [];
        }
        let keys = [];
        const { schema } = datasource;
        if (!(schema === null || schema === void 0 ? void 0 : schema.properties))
            return [];
        if (isUseAll) {
            keys = Object.keys(schema.properties || {});
        }
        else {
            // 过滤掉系统字段 x-system=true 和不存在 datasource 中的字段（被删除的字段）
            keys = enabledFieldNames.filter((key) => {
                const field = schema.properties[key];
                return field && (displaySystem || !isSystemField(field));
            });
        }
        // 添加关联数据源
        const resultPromise = keys.map(async (key) => {
            var _a;
            let parentDatasource;
            if ((_a = schema.properties[key]) === null || _a === void 0 ? void 0 : _a['x-parent']) {
                parentDatasource =
                    (await getDataSourceByName(schema.properties[key]['x-parent'].parentDataSourceName, true)) || {};
            }
            return {
                ...schema.properties[key],
                name: key,
                parentDatasource,
            };
        });
        const result = await Promise.all(resultPromise);
        return getDataSourceFieldsWithAuth(result, datasource === null || datasource === void 0 ? void 0 : datasource.resourceCheckInfos);
    }
    else {
        if (!connectorMethod)
            return [];
        const datasource = await getDataSourceByName(dbName);
        // 警告
        if (!datasource) {
            console.error(`未找到数据源 ${dbName}`, window.app.cloud);
            return [];
        }
        const connectKeys = [];
        const { methods } = datasource;
        const { name } = connectorMethod;
        let method = [];
        if (methods && methods.length > 0) {
            method = methods.filter((item) => {
                return item.name == name;
            });
        }
        datasource.schema = ((_d = (_c = (_b = (_a = method[0]) === null || _a === void 0 ? void 0 : _a.outParams) === null || _b === void 0 ? void 0 : _b.properties) === null || _c === void 0 ? void 0 : _c.records) === null || _d === void 0 ? void 0 : _d.items) || {};
        const { schema } = datasource;
        if (!(schema === null || schema === void 0 ? void 0 : schema.properties))
            return [];
        for (const item in schema.properties) {
            connectKeys.push(item);
        }
        // 添加关联数据源
        const resultPromise = connectKeys.map(async (key) => {
            var _a;
            let parentDatasource;
            if ((_a = schema.properties[key]) === null || _a === void 0 ? void 0 : _a['x-parent']) {
                parentDatasource = (await getDataSourceByName(schema.properties[key]['x-parent'].parentDataSourceName)) || {};
            }
            return {
                ...schema.properties[key],
                name: key,
                parentDatasource,
            };
        });
        const result = await Promise.all(resultPromise);
        return result === null || result === void 0 ? void 0 : result.map((item) => ({ ...item, readWriteAuth: 'rw' }));
    }
};
/**
 * 获取主列字段标识并展示主列字段
 */
export const getPrimaryField = (data) => {
    const keys = Object.keys(data).filter((item) => item.includes('@'));
    keys.forEach((item) => {
        var _a, _b, _c, _d, _e;
        const field = item.replace('@', '');
        const record = (_a = data[item]) === null || _a === void 0 ? void 0 : _a.v1.record;
        const name = (_b = data[item]) === null || _b === void 0 ? void 0 : _b.v1.primaryColumn;
        if (name) {
            // 系统主列字段当record不存在时，默认展示最外层owner、createBy、updateBy字段，与其他列字段逻辑一致
            if (record && record._id === data[field]) {
                // 当record下主列字段没有值时，默认返回主列id
                record[name] ? (data[field] = record[name]) : (data[field] = ((_e = (_d = (_c = data[item]) === null || _c === void 0 ? void 0 : _c.v1) === null || _d === void 0 ? void 0 : _d.record) === null || _e === void 0 ? void 0 : _e._id) || '-');
            }
        }
    });
    return data;
};
